@@ -812,6 +812,9 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 	double totalscore;
 	double dumdb = 0.0;
 	int headgp, tailgp;
+	static TLS double *gstart = NULL;
+	static TLS double *gend = NULL;
+	static TLS double **codonscoremtx;
 
 
 	if( seq1 == NULL )
@@ -829,6 +832,7 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 			A__align_variousdist( NULL, NULL, NULL, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
 			D__align_variousdist( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
 			G__align11( NULL, NULL, NULL, 0, 0, 0 );
+			G__align11psg( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL );
 			blockAlign2( NULL, NULL, NULL, NULL, NULL, NULL );
 			if( crossscore ) FreeDoubleMtx( crossscore );
 			crossscore = NULL;
@@ -841,6 +845,9 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 			free( segment2 );
 			free( sortedseg1 );
 			free( sortedseg2 );
+			if( gstart ) free( gstart );
+			if( gend ) free( gend );
+			if( codonscoremtx ) FreeDoubleMtx( codonscoremtx );
 			if( !kobetsubunkatsu )
 			{
 				FreeFukusosuuMtx ( seqVector1 );
@@ -858,6 +865,7 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 		return( 0.0 );
 	}
 
+
 	len1 = strlen( seq1[0] );
 	len2 = strlen( seq2[0] );
 	nlentmp = MAX( len1, len2 );
@@ -875,6 +883,7 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 	fprintf( stderr, "nlentmp = %d, nlen = %d\n", nlentmp, nlen );
 #endif
 
+
 	result1 = AllocateCharMtx( clus1, alloclen );
 	result2 = AllocateCharMtx( clus2, alloclen );
 	tmpres1 = AllocateCharMtx( clus1, alloclen );
@@ -914,7 +923,148 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 		if     ( scoremtx == -1 ) n20or4or2 = 1;
 		else if( fftscore )       n20or4or2 = 1;
 		else                      n20or4or2 = 20;
+
+
+		gstart = NULL;
+		gend = NULL;
+	 	if( codonpos ) // gstart to gend ha sudeni allocate sareteiru kamo
+		{
+			FILE *cfp;
+			int p;
+			char *buf = calloc( sizeof( char ), 1000 );
+
+			if( dorp != 'd' )
+			{
+				reporterr( "\n\nThe --codonpos and --codonscore options are only for DNA data.\n\n" );
+				exit( 1 );
+			}
+
+//			reporterr( "\nIn Falign, loading position-specific gap costs\n" );
+			gstart = calloc( sizeof(double), len1+1 );
+			for( i=0; i<len1+1; i++ ) gstart[i] = 1.0 * 0.5; // init
+			gend = calloc( sizeof(double), len1+1 );
+			for( i=0; i<len1+1; i++ ) gend[i] = 1.0 * 0.5; // init
+#define STRONG 3.0
+
+			i = 0;
+			cfp = fopen( "_codonpos", "r" );	
+			if( cfp == NULL )
+			{
+				reporterr( "Cannot open _codonpos file\n" );
+				exit( 1 );
+			}
+			while( fgets( buf, 1000, cfp ) )
+			{
+				if( i == len1 )
+				{
+					reporterr( "\n\nNumber of lines in the codonposition file must be the same as the length of the first sequences (%d).\n\n", len1 );
+					exit( 1 );
+				}
+
+				if( buf[0] == '#' )
+				{
+					continue;
+				}
+				else if( buf[0] == '0' )
+				{
+					gstart[i] = 1.0 * 0.5; // noncoding
+					gend[i] = 1.0 * 0.5; // noncoding
+				}
+				else if( buf[0] == '1' )
+				{
+					gstart[i] = STRONG * 0.5;
+					gend[i] = 1.0 * 0.5;
+				}
+				else if( buf[0] == '2' )
+				{
+					gstart[i] = STRONG * 0.5;
+					gend[i] = STRONG * 0.5;
+				}
+				else if( buf[0] == '3' )
+				{
+					gstart[i] = 1.0 * 0.5;
+					gend[i] = STRONG * 0.5;
+				}
+				else
+				{
+					reporterr( "In the codonposition file, 1st letter in a line must be either of:\n" );
+					reporterr( "	0 (noncoding)\n" );
+					reporterr( "	1 (1st position in codon)\n" );
+					reporterr( "	2 (2nd position in codon)\n" );
+					reporterr( "	3 (3rd position in codon)\n" );
+					reporterr( "When mutliple difference frames are used, set 2.\n" );
+					exit( 1 );
+				}
+				i++;
+			}
+			fclose( cfp );
+			free( buf );
+			if( i < len1 )
+			{
+				reporterr( "\n\nNumber of lines in the codonposition file (%d) is less than the length of the first sequences (%d).\n\n", i, len1 );
+				exit( 1 );
+			}
+
+		}
+
+		if( codonscore )
+		{
+			int i, j;
+			FILE *cfp;
+			double *codonfreq = calloc( sizeof( double ), 64 );
+			double totalcount, codonscore0av, codonscore1av;
+			if( !codonpos )
+			{
+				reporterr( "\n\n --codonpos is necessary for --codonscore\n\n" );
+				exit( 1 );
+			}
+			codonscoremtx = AllocateDoubleMtx( 64, 64 );
+			cfp = fopen( "_codonscore", "r" );	
+			loadcodonscore( cfp, codonscoremtx );
+			fclose( cfp );
+			for( i=0; i<64; i++ ) codonfreq[i] = 0.0;
+			totalcount = 0.0;
+			for( i=3; i<len1; i++ )
+			{
+//				reporterr( "i=%d\n", i );
+				if( gstart[i-2] == STRONG * 0.5 && gend[i-2] == 1.0 * 0.5    && 
+				    gstart[i-1] == STRONG * 0.5 && gend[i-1] == STRONG * 0.5 &&
+				    gstart[i-0] == 1.0 * 0.5    && gend[i-0] == STRONG * 0.5 )
+				{
+//					reporterr( "codon=%.3s, id=%d\n", seq1[0]+i-2, codon2id(seq1[0]+i-2) );
+					codonfreq[codon2id(seq1[0]+i-2)] += 1.0;
+					totalcount += 1.0;
+				}
+			}
+			for( i=0; i<64; i++ ) codonfreq[i] /= totalcount;
+//			for( i=0; i<64; i++ ) reporterr( "%d, %f\n", i, codonfreq[i] );
+
+			codonscore0av = 0.0;
+			for( i=0; i<64; i++ ) for( j=0; j<64; j++ ) codonscore0av += codonfreq[i]*codonfreq[j] * codonscoremtx[i][j];
+//			reporterr( "0av=%f\n", codonscore0av );
+			for( i=0; i<64; i++ ) for( j=0; j<64; j++ ) codonscoremtx[i][j] -= codonscore0av;
+
+			codonscore1av = 0.0;
+			for( i=0; i<64; i++ ) codonscore1av += codonfreq[i] * codonscoremtx[i][i];
+//			reporterr( "1av=%f\n", codonscore1av );
+			for( i=0; i<64; i++ ) for( j=0; j<64; j++ ) codonscoremtx[i][j] /= codonscore1av;
+
+#if 0
+			codonscore1av = 0.0;
+			for( i=0; i<64; i++ ) codonscore1av += codonfreq[i] * codonscoremtx[i][i];
+			reporterr( "1av=%f\n", codonscore1av );
+
+			codonscore0av = 0.0;
+			for( i=0; i<64; i++ ) for( j=0; j<64; j++ ) codonscore0av += codonfreq[i]*codonfreq[j] * codonscoremtx[i][j];
+			reporterr( "0av=%f\n", codonscore0av );
+#endif
+
+			free( codonfreq );
+		}
+		else
+			codonscoremtx = NULL;
 	}
+
 	if( localalloclen < nlen )
 	{
 		if( localalloclen )
@@ -1364,6 +1514,7 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 	for( j=0; j<clus2; j++ ) result2[j][0] = 0;
 	totalscore = 0.0;
 	*fftlog = -1;
+//	reporterr( "\nin Falign(), *fftlog = %d\n", *fftlog );
 	for( i=0; i<count-1; i++ )
 	{
 		*fftlog += 1;
@@ -1530,10 +1681,21 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 			case( 'A' ):
 				if( clus1 == 1 && clus2 == 1 )
 				{
-					totalscore += G__align11( n_dynamicmtx, tmpres1, tmpres2, alloclen, headgp, tailgp );
+					if( codonpos || codonscore )
+					{
+//						reporterr( "calling G__align11psg\n" );
+						totalscore += G__align11psg( codonscoremtx, n_dynamicmtx, tmpres1, tmpres2, alloclen, headgp, tailgp, gstart+cut1[i], gend+cut1[i] );
+					}
+					else
+						totalscore += G__align11( n_dynamicmtx, tmpres1, tmpres2, alloclen, headgp, tailgp );
 				}
 				else
 				{
+					if( codonpos )
+					{
+						reporterr( "\n\ncodonpos will be soon supported for a reference MSA. For now, use a single sequence as reference.\n\n\n" );
+						exit( 1 );
+					}
 					if( scoringmatrices ) // called by tditeration.c
 					{
 						totalscore += A__align_variousdist( whichmtx, scoringmatrices, NULL, penalty, penalty_ex, tmpres1, tmpres2, eff1, eff2, eff1s, eff2s, clus1, clus2, alloclen, 0, &dumdb, sgap1, sgap2, egap1, egap2, chudanpt, chudanref, chudanres, headgp, tailgp );
@@ -1552,6 +1714,23 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 		if( chudanres && *chudanres )
 		{
 //			fprintf( stderr, "\n\n## CHUUDAN!!! at Falign_localhom\n" );
+//			Added 2021/Jul/25.
+			FreeCharMtx( result1 );
+			FreeCharMtx( result2 );
+			FreeCharMtx( tmpres1 );
+			FreeCharMtx( tmpres2 );
+			FreeCharMtx( tmpseq1 );
+			FreeCharMtx( tmpseq2 );
+			free( sgap1 );
+			free( egap1 );
+			free( sgap2 );
+			free( egap2 );
+			free( tmpptr1 );
+			free( tmpptr2 );
+#if RND
+			FreeCharMtx( rndseq1 );
+			FreeCharMtx( rndseq2 );
+#endif
 			return( -1.0 );
 		}
 #endif
@@ -1580,6 +1759,7 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 		}
 #endif
 	}
+//	reporterr( "\nafter Falign(), *fftlog = %d\n", *fftlog );
 
 #if KEIKA
 	fprintf( stderr, "DP ... done   \n" );
@@ -1654,9 +1834,9 @@ static double estimategapfreq( int n, char **s )
 	return( fv/(double)n );
 }
 
-static int terminalmargin( int lshorter, double groupsizefac )
+static int getterminalmargin( int lshorter, double groupsizefac )
 {
-	return ( lshorter * 2.0 + 100 ) * groupsizefac;
+	return ( lshorter * 2.0 + terminalmargin ) * groupsizefac;
 //	return ( lshorter * 1.1 + 10 ) * groupsizefac;
 }
 
@@ -1881,9 +2061,9 @@ double Falign_givenanchors( ExtAnch *pairanch,
 		sgap2 = AllocateCharVec( njob );
 		egap2 = AllocateCharVec( njob );
 		//kouho = AllocateIntVec( NKOUHO_LONG );
-		alignorcopy = AllocateIntVec( MAXSEG );
-		cut1 = AllocateIntVec( MAXSEG );
-		cut2 = AllocateIntVec( MAXSEG );
+		alignorcopy = AllocateIntVec( MAXSEG_GIVENANCHORS );
+		cut1 = AllocateIntVec( MAXSEG_GIVENANCHORS );
+		cut2 = AllocateIntVec( MAXSEG_GIVENANCHORS );
 		//tmpptr1 = AllocateCharMtx( njob, 0 );
 		//tmpptr2 = AllocateCharMtx( njob, 0 );
 		//segment = (Segment *)calloc( MAXSEG, sizeof( Segment ) );
@@ -1943,359 +2123,6 @@ double Falign_givenanchors( ExtAnch *pairanch,
 		localalloclen = nlen;
 	}
 	
-#if 0
-	for( j=0; j<clus1; j++ ) strcpy( tmpseq1[j], seq1[j] );
-	for( j=0; j<clus2; j++ ) strcpy( tmpseq2[j], seq2[j] );
-
-	if( !kobetsubunkatsu )
-	{
-		if( fftkeika ) fprintf( stderr,  " FFT ... " );
-
-		for( j=0; j<n20or4or2; j++ ) vec_init( seqVector1[j], nlen );
-		if( scoremtx == -1 )
-		{
-			for( i=0; i<clus1; i++ )
-				seq_vec_4( seqVector1[0], eff1[i], tmpseq1[i] );
-		}
-		else if( fftscore )
-		{
-			for( i=0; i<clus1; i++ )
-			{
-#if 0
-				seq_vec_2( seqVector1[0], polarity, eff1[i], tmpseq1[i] );
-				seq_vec_2( seqVector1[1], volume,   eff1[i], tmpseq1[i] );
-#else
-				seq_vec_5( seqVector1[0], polarity, volume, eff1[i], tmpseq1[i] );
-#endif
-			}
-		}
-		else
-		{
-			for( i=0; i<clus1; i++ )
-				seq_vec_3( seqVector1, eff1[i], tmpseq1[i] );
-		}
-#if RND
-		for( i=0; i<clus1; i++ )
-		{
-			vec_init2( seqVector1, rndseq1[i], eff1[i], len1, nlen );
-		}
-#endif
-
-		for( j=0; j<n20or4or2; j++ ) vec_init( seqVector2[j], nlen );
-		if( scoremtx == -1 )
-		{
-			for( i=0; i<clus2; i++ )
-				seq_vec_4( seqVector2[0], eff2[i], tmpseq2[i] );
-		}
-		else if( fftscore )
-		{
-			for( i=0; i<clus2; i++ )
-			{
-#if 0
-				seq_vec_2( seqVector2[0], polarity, eff2[i], tmpseq2[i] );
-				seq_vec_2( seqVector2[1], volume,   eff2[i], tmpseq2[i] );
-#else
-				seq_vec_5( seqVector2[0], polarity, volume, eff2[i], tmpseq2[i] );
-#endif
-			}
-		}
-		else
-		{
-			for( i=0; i<clus2; i++ )
-				seq_vec_3( seqVector2, eff2[i], tmpseq2[i] );
-		}
-#if RND
-		for( i=0; i<clus2; i++ )
-		{
-			vec_init2( seqVector2, rndseq2[i], eff2[i], len2, nlen );
-		}
-#endif
-
-
-		for( j=0; j<n20or4or2; j++ )
-		{
-			fft( nlen, seqVector2[j], 0 );
-			fft( nlen, seqVector1[j], 0 );
-		}
-
-		for( k=0; k<n20or4or2; k++ ) 
-		{
-			for( l=0; l<nlen; l++ ) 
-				calcNaiseki( naiseki[k]+l, seqVector1[k]+l, seqVector2[k]+l );
-		}
-		for( l=0; l<nlen; l++ ) 
-		{
-			naisekiNoWa[l].R = 0.0;
-			naisekiNoWa[l].I = 0.0;
-			for( k=0; k<n20or4or2; k++ ) 
-			{
-				naisekiNoWa[l].R += naiseki[k][l].R;
-				naisekiNoWa[l].I += naiseki[k][l].I;
-			}
-		}
-	
-
-		fft( -nlen, naisekiNoWa, 0 );
-	
-		for( m=0; m<=nlen2; m++ ) 
-			soukan[m] = naisekiNoWa[nlen2-m].R;
-		for( m=nlen2+1; m<nlen; m++ ) 
-			soukan[m] = naisekiNoWa[nlen+nlen2-m].R;
-
-
-		nkouho = getKouho( kouho, NKOUHO_LONG, soukan, nlen );
-
-#if 0
-		for( i=0; i<nkouho; i++ )
-		{
-			fprintf( stderr, "kouho[%d] = %d\n", i, kouho[i] );
-		}
-#endif
-	}
-
-#if KEIKA
-	fprintf( stderr, "Searching anchors ... " );
-#endif
-	count = 0;
-
-
-
-	if( kobetsubunkatsu )
-	{
-		maxk = 1;
-		kouho[0] = 0;
-	}
-	else
-	{
-		maxk = nkouho;
-	}
-
-	for( k=0; k<maxk; k++ ) 
-	{
-		lag = kouho[k];
-		if( lag <= -len1 || len2 <= lag ) continue;
-//		fprintf( stderr, "k=%d, lag=%d\n", k, lag );
-		zurasu2( lag, clus1, clus2, seq1, seq2, tmpptr1, tmpptr2 );
-
-//		fprintf( stderr, "lag = %d\n", lag );
-		tmpint = alignableReagion( clus1, clus2, tmpptr1, tmpptr2, eff1, eff2, segment+count );
-//		fprintf( stderr, "lag = %d, %d found\n", lag, tmpint );
-
-//		if( lag == -50 ) exit( 1 );
-		
-		if( count+tmpint > MAXSEG -3 ) ErrorExit( "TOO MANY SEGMENTS.\n" );
-
-//		fprintf( stderr, "##### k=%d / %d\n", k, maxk );
-//		if( tmpint == 0 ) break; // 060430 iinoka ? // 090530 yameta
-		while( tmpint-- > 0 )
-		{
-#if 0
-			if( segment[count].end - segment[count].start < fftWinSize )
-			{
-				count++;
-				continue;
-			}
-#endif
-			if( lag > 0 )
-			{
-				segment1[count].start  = segment[count].start ;
-				segment1[count].end    = segment[count].end   ;
-				segment1[count].center = segment[count].center;
-				segment1[count].score  = segment[count].score;
-
-				segment2[count].start  = segment[count].start  + lag;
-				segment2[count].end    = segment[count].end    + lag;
-				segment2[count].center = segment[count].center + lag;
-				segment2[count].score  = segment[count].score       ;
-			}
-			else
-			{
-				segment1[count].start  = segment[count].start  - lag;
-				segment1[count].end    = segment[count].end    - lag;
-				segment1[count].center = segment[count].center - lag;
-				segment1[count].score  = segment[count].score       ;
-
-				segment2[count].start  = segment[count].start ;
-				segment2[count].end    = segment[count].end   ;
-				segment2[count].center = segment[count].center;
-				segment2[count].score  = segment[count].score ;
-			}
-#if 0
-			fprintf( stderr, "##### k=%d / %d\n", k, maxk );
-			fprintf( stderr, "anchor %d, score = %f\n", count, segment1[count].score );
-			fprintf( stderr, "in 1 %d\n", segment1[count].center );
-			fprintf( stderr, "in 2 %d\n", segment2[count].center );
-#endif
-			segment1[count].pair = &segment2[count];
-			segment2[count].pair = &segment1[count];
-			count++;
-#if 0
-			fprintf( stderr, "count=%d\n", count );
-#endif
-		}
-	}
-#if 1
-	if( !kobetsubunkatsu )
-		if( fftkeika ) fprintf( stderr, "done. (%d anchors) ", count );
-#endif
-	if( !count && fftNoAnchStop )
-		ErrorExit( "Cannot detect anchor!" );
-#if 0
-	fprintf( stderr, "RESULT before sort:\n" );
-	for( l=0; l<count+1; l++ )
-	{
-		fprintf( stderr, "cut[%d]=%d, ", l, segment1[l].center );
-		fprintf( stderr, "%d score = %f\n", segment2[l].center, segment1[l].score );
-	}
-#endif
-
-	for( i=0; i<count; i++ )
-	{
-		sortedseg1[i] = &segment1[i];
-		sortedseg2[i] = &segment2[i];
-	}
-#if 0
-	tmpsort( count, sortedseg1 ); 
-	tmpsort( count, sortedseg2 ); 
-	qsort( sortedseg1, count, sizeof( Segment * ), segcmp );
-	qsort( sortedseg2, count, sizeof( Segment * ), segcmp );
-#else
-	mymergesort( 0, count-1, sortedseg1 ); 
-	mymergesort( 0, count-1, sortedseg2 ); 
-#endif
-	for( i=0; i<count; i++ ) sortedseg1[i]->number = i;
-	for( i=0; i<count; i++ ) sortedseg2[i]->number = i;
-
-
-
-	if( kobetsubunkatsu )
-	{
-		for( i=0; i<count; i++ )
-	    {
-			cut1[i+1] = sortedseg1[i]->center;
-			cut2[i+1] = sortedseg2[i]->center;
-		}
-		cut1[0] = 0;
-		cut2[0] = 0;
-		cut1[count+1] = len1;
-		cut2[count+1] = len2;
-		count += 2;
-	}
-
-	else
-	{
-		if( count < 5000 )
-		{
-			if( crossscoresize < count+2 )
-			{
-				crossscoresize = count+2;
-#if 1
-				if( fftkeika ) fprintf( stderr, "######allocating crossscore, size = %d\n", crossscoresize );
-#endif
-				if( crossscore ) FreeDoubleMtx( crossscore );
-				crossscore = AllocateDoubleMtx( crossscoresize, crossscoresize );
-			}
-			for( i=0; i<count+2; i++ ) for( j=0; j<count+2; j++ )
-				crossscore[i][j] = 0.0;
-			for( i=0; i<count; i++ )
-			{
-				crossscore[segment1[i].number+1][segment1[i].pair->number+1] = segment1[i].score;
-				cut1[i+1] = sortedseg1[i]->center;
-				cut2[i+1] = sortedseg2[i]->center;
-			}
-	
-#if 0
-			fprintf( stderr, "AFTER SORT\n" );
-			for( i=0; i<count+1; i++ ) fprintf( stderr, "%d, %d\n", cut1[i], cut2[i] );
-			fprintf( stderr, "crossscore = \n" );
-			for( i=0; i<count+1; i++ )
-			{
-				for( j=0; j<count+1; j++ )
-					fprintf( stderr, "%.0f ", crossscore[i][j] );
-				fprintf( stderr, "\n" );
-			}
-#endif
-
-			crossscore[0][0] = 10000000.0;
-			cut1[0] = 0; 
-			cut2[0] = 0;
-			crossscore[count+1][count+1] = 10000000.0;
-			cut1[count+1] = len1;
-			cut2[count+1] = len2;
-			count += 2;
-			count0 = count;
-		
-//			fprintf( stderr, "\n\n\ncalling blockAlign2\n\n\n\n" );
-			blockAlign2( cut1, cut2, sortedseg1, sortedseg2, crossscore, &count );
-	
-//			if( count-count0 )
-//				fprintf( stderr, "%d unused anchors\n", count0-count );
-	
-			if( !kobetsubunkatsu && fftkeika )
-				fprintf( stderr, "%d anchors found\n", count );
-			if( fftkeika )
-			{
-				if( count0 > count )
-				{
-#if 0
-					fprintf( stderr, "\7 REPEAT!? \n" ); 
-#else
-					fprintf( stderr, "REPEAT!? \n" ); 
-#endif
-					if( fftRepeatStop ) exit( 1 );
-				}
-#if KEIKA
-				else fprintf( stderr, "done\n" );
-#endif
-			}
-		}
-
-
-		else
-		{
-			fprintf( stderr, "\nMany anchors were found. The upper-level DP is skipped.\n\n" );
-
-			cut1[0] = 0; 
-			cut2[0] = 0;
-			count0 = 0;
-			for( i=0; i<count; i++ )
-			{
-//				fprintf( stderr, "i=%d, %d-%d ?\n", i, sortedseg1[i]->center, sortedseg1[i]->pair->center );
-				if( sortedseg1[i]->center > cut1[count0]
-				 && sortedseg1[i]->pair->center > cut2[count0] )
-				{
-					count0++;
-					cut1[count0] = sortedseg1[i]->center;
-					cut2[count0] = sortedseg1[i]->pair->center;
-				}
-				else
-				{
-					if( i && sortedseg1[i]->score > sortedseg1[i-1]->score )
-					{
-						if( sortedseg1[i]->center > cut1[count0-1]
-						 && sortedseg1[i]->pair->center > cut2[count0-1] )
-						{
-							cut1[count0] = sortedseg1[i]->center;
-							cut2[count0] = sortedseg1[i]->pair->center;
-						}
-						else
-						{
-//							count0--;
-						}
-					}
-				}
-			}
-//			if( count-count0 )
-//				fprintf( stderr, "%d anchors unused\n", count-count0 );
-			cut1[count0+1] = len1;
-			cut2[count0+1] = len2;
-			count = count0 + 2;
-			count0 = count;
-	
-		}
-	}
-	//uwagaki!
-#endif
 
 	marginfac1 = 1.0 + estimategapfreq( clus1, seq1 );
 	marginfac2 = 1.0 + estimategapfreq( clus2, seq2 );
@@ -2319,20 +2146,20 @@ double Falign_givenanchors( ExtAnch *pairanch,
 			count0++;
 			continue;
 		}
-		if( count+2 > MAXSEG -3 ) ErrorExit( "TOO MANY SEGMENTS.\n" );
+		if( count+2 > MAXSEG_GIVENANCHORS -3 ) ErrorExit( "TOO MANY SEGMENTS.\n" );
 #if 1 // mattan no tansaku hann'i wo seigen
 		if( count == 0 ) 	
 		{
 //			if( pairanch[count0].starti - pairanch[count0].startj > TERMINALSEGMENTLENGTH ) // you kentou
 //			nogaplen1 = estimatenogaplen( clus1, seq1, pairanch[count0].starti, 0 );
 //			nogaplen2 = estimatenogaplen( clus2, seq2, pairanch[count0].startj, 0 );
-			if( pairanch[count0].starti > terminalmargin(pairanch[count0].startj,marginfac1) )
+			if( pairanch[count0].starti > getterminalmargin(pairanch[count0].startj,marginfac1) )
 			{
 //				alignorcopy[1] = 'A';
-//				reporterr( "check 1, because starti=%d > startj=%d -> %d (clus1=%d)\n", pairanch[count0].starti, pairanch[count0].startj, terminalmargin(pairanch[count0].startj,marginfac1), clus1 );
-				cutadd = pairanch[count0].starti - terminalmargin(pairanch[count0].startj,marginfac1);
-//				reporterr( "cutadd(1)=%d\n", cutadd );
-//				if( 1 || cutadd > TERMINALMARGIN(0) ) // iranai
+				reporterr( "check 1, because starti=%d > startj=%d -> %d (clus1=%d)\n", pairanch[count0].starti, pairanch[count0].startj, getterminalmargin(pairanch[count0].startj,marginfac1), clus1 );
+				cutadd = pairanch[count0].starti - getterminalmargin(pairanch[count0].startj,marginfac1);
+				reporterr( "cutadd(1)=%d\n", cutadd );
+//				if( 0 || cutadd > TERMINALMARGIN(0) ) // iranai
 				{
 					cut1[1] = cutadd;
 					cut2[1] = 0;
@@ -2341,12 +2168,12 @@ double Falign_givenanchors( ExtAnch *pairanch,
 					starttermcut1 = 1;
 				}
 			}
-			else if( pairanch[count0].startj > terminalmargin(pairanch[count0].starti, marginfac2) )
+			else if( pairanch[count0].startj > getterminalmargin(pairanch[count0].starti, marginfac2) )
 			{
 //				alignorcopy[1] = 'A';
-//				reporterr( "check 2, because startj=%d > starti=%d -> %d (clus2=%d)\n", pairanch[count0].startj, pairanch[count0].starti, terminalmargin(pairanch[count0].starti,marginfac2), clus2 );
-				cutadd = pairanch[count0].startj - terminalmargin( pairanch[count0].starti, marginfac2 );
-//				reporterr( "cutadd(2)=%d\n", cutadd );
+				reporterr( "check 2, because startj=%d > starti=%d -> %d (clus2=%d)\n", pairanch[count0].startj, pairanch[count0].starti, getterminalmargin(pairanch[count0].starti,marginfac2), clus2 );
+				cutadd = pairanch[count0].startj - getterminalmargin( pairanch[count0].starti, marginfac2 );
+				reporterr( "cutadd(2)=%d\n", cutadd );
 				{
 					cut1[1] = 0;
 					cut2[1] = cutadd;
@@ -2365,12 +2192,13 @@ double Falign_givenanchors( ExtAnch *pairanch,
 		count += 1;
 
 #if 1
+#define DIVIDEANCHOR 100
 		if( pairanch[count0].endi - cut1[count] == pairanch[count0].endj - cut2[count] )
-		while( pairanch[count0].endi+1 - cut1[count] > 100 && pairanch[count0].endj+1 - cut2[count] > 100 )
+		while( pairanch[count0].endi+1 - cut1[count] > DIVIDEANCHOR && pairanch[count0].endj+1 - cut2[count] > DIVIDEANCHOR )
 		{
-			reporterr( "added an anchor, because the length is %d,%d > 100\n", pairanch[count0].endi+1 - cut1[count], pairanch[count0].endj+1 - cut2[count] );
-			cut1[count+1] = cut1[count] + 100;
-			cut2[count+1] = cut2[count] + 100;
+			reporterr( "added an anchor, because the length is %d,%d > DIVIDEANCHOR.    \r", pairanch[count0].endi+1 - cut1[count], pairanch[count0].endj+1 - cut2[count] );
+			cut1[count+1] = cut1[count] + DIVIDEANCHOR;
+			cut2[count+1] = cut2[count] + DIVIDEANCHOR;
 			alignorcopy[count+1] = 'c';
 			count += 1;
 		}
@@ -2385,11 +2213,12 @@ double Falign_givenanchors( ExtAnch *pairanch,
 		count += 1;
 		count0++;
 	}
+	reporterr( "\n" );
 
 #if 1 // mattan no tansaku hanni wo seigen
 	alignorcopy[count] = 'a';
 //	if( count > 1 && (len1-cut1[count]) > (len2-cut2[count]) + 2*TERMINALSEGMENTLENGTH ) // 2 ha tekitou
-	if( count > 1 && (len1-cut1[count]) > terminalmargin(len2-cut2[count],marginfac1) )
+	if( count > 1 && (len1-cut1[count]) > getterminalmargin(len2-cut2[count],marginfac1) )
 	{
 //		reporterr( "last\n" );
 //		alignorcopy[count] = 'A'; // mae no wo uwagaki
@@ -2398,7 +2227,7 @@ double Falign_givenanchors( ExtAnch *pairanch,
 //		cut1[count+1] = cut1[count] + TERMINALSEGMENTLENGTH;
 //		cut1[count+1] = cut1[count] + (len2-cut2[count]) + TERMINALSEGMENTLENGTH;
 
-		cutadd = len1 - 1 - ( (len1-cut1[count]) - terminalmargin(len2-cut2[count], marginfac1) ); // wakarinikuikedo
+		cutadd = len1 - 1 - ( (len1-cut1[count]) - getterminalmargin(len2-cut2[count], marginfac1) ); // wakarinikuikedo
 
 //		if( 1 || len1-1 - cutadd > TERMINALMARGIN(0) )
 		{
@@ -2414,13 +2243,13 @@ double Falign_givenanchors( ExtAnch *pairanch,
 		}
 	}
 //	else if( count > 1 && (len2-cut2[count]) > (len1-cut1[count])  + 2*TERMINALSEGMENTLENGTH ) // 2 ha tekitou
-	else if( count > 1 && (len2-cut2[count]) > terminalmargin(len1-cut1[count],marginfac2) )
+	else if( count > 1 && (len2-cut2[count]) > getterminalmargin(len1-cut1[count],marginfac2) )
 	{
 //		reporterr( "last\n" );
 //		alignorcopy[count] = 'A'; // mae no wo uwagaki
 		//reporterr( "insert one anchor to restrict terminal gap length, 2, cut1[count]=%d, cut2[count]=%d\n", cut1[count], cut2[count] );
 		//alignorcopy[count] = 'A'; // mae no wo uwagaki
-		cutadd = len2 - 1 - ( (len2-cut2[count]) - terminalmargin(len1-cut1[count], marginfac2) );
+		cutadd = len2 - 1 - ( (len2-cut2[count]) - getterminalmargin(len1-cut1[count], marginfac2) );
 
 //		if( 1 || len2-1 - cutadd > TERMINALMARGIN(0) ) // iranai
 		{
@@ -2479,7 +2308,7 @@ double Falign_givenanchors( ExtAnch *pairanch,
 //exit( 1 );
 	for( i=0; i<count-1; i++ )
 	{
-		//reporterr( "\ni=%d / %d \n\n", i, count );
+//		reporterr( "\ni=%d / %d \n\n", i, count );
 		*fftlog += 1;
 		if( i == 0 || ( i == 1 && alignorcopy[1] == 'A' ) ) headgp = outgap; else headgp = 1;
 		if( i == count-2 || ( i == count-3 && alignorcopy[count-3] == 'A' ) ) tailgp = outgap; else tailgp = 1;
@@ -2535,7 +2364,7 @@ double Falign_givenanchors( ExtAnch *pairanch,
 		fprintf( stderr, "DP %03d / %03d %4d to ", i+1, count-1, totallen );
 #else
 #if 1
-		if( 1 || fftkeika ) fprintf( stderr, "DP %05d / %05d \b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b", i+1, count-1 );
+		if( 1 || fftkeika ) fprintf( stderr, "DP %05d / %05d \r", i+1, count-1 );
 #endif
 #endif
 		for( j=0; j<clus1; j++ )
@@ -2663,12 +2492,12 @@ double Falign_givenanchors( ExtAnch *pairanch,
 			if( starttermcut1 )
 			{
 				for( j=0; j<clus2; j++ ) if( tmpres2[j][0] != '-' ) break;
-				if( j<clus2 ) reporterr( "There may be a problem in a hard-coded parameter (1).  Please contact katoh@ifrec.osaka-u.ac.jp\n" );
+				if( j<clus2 ) reporterr( "There may be a problem at the 5' end (1). Please contact katoh@ifrec.osaka-u.ac.jp\n", tmpres2[j] );
 			}
 			else if( starttermcut2 )
 			{
 				for( j=0; j<clus1; j++ ) if( tmpres1[j][0] != '-' ) break;
-				if( j<clus1 ) reporterr( "There may be a problem in a hard-coded parameter (2).  Please contact katoh@ifrec.osaka-u.ac.jp\n" );
+				if( j<clus1 ) reporterr( "There may be a problem at the 5' end (2).  Please contact katoh@ifrec.osaka-u.ac.jp\n" );
 			}
 			else
 				break;
@@ -2680,13 +2509,13 @@ double Falign_givenanchors( ExtAnch *pairanch,
 			{
 				tmplen = strlen( tmpres2[0] );
 				for( j=0; j<clus2; j++ ) if( tmpres2[j][tmplen-1] != '-' ) break;
-				if( j<clus2 ) reporterr( "There may be a problem in a hard-coded parameter (3).  Please contact katoh@ifrec.osaka-u.ac.jp\n" );
+				if( j<clus2 ) reporterr( "There may be a problem at the 3' end (3).  Please contact katoh@ifrec.osaka-u.ac.jp\n" );
 			}
 			else if( endtermcut2 )
 			{
 				tmplen = strlen( tmpres1[0] );
 				for( j=0; j<clus1; j++ ) if( tmpres1[j][tmplen-1] != '-' ) break;
-				if( j<clus1 ) reporterr( "There may be a problem in a hard-coded parameter (4).  Please contact katoh@ifrec.osaka-u.ac.jp\n" );
+				if( j<clus1 ) reporterr( "There may be a problem at the 3' end (4).  Please contact katoh@ifrec.osaka-u.ac.jp\n" );
 			}
 			else
 				break;
@@ -2699,8 +2528,13 @@ double Falign_givenanchors( ExtAnch *pairanch,
 			fprintf( stderr, "totallen=%d +  nlen=%d > alloclen = %d\n", totallen, nlen, alloclen );
 			ErrorExit( "LENGTH OVER in Falign\n " );
 		}
+#if 0
 		for( j=0; j<clus1; j++ ) strcat( result1[j], tmpres1[j] );
 		for( j=0; j<clus2; j++ ) strcat( result2[j], tmpres2[j] );
+#else
+		for( j=0; j<clus1; j++ ) strcat( result1[j]+totallen, tmpres1[j] );
+		for( j=0; j<clus2; j++ ) strcat( result2[j]+totallen, tmpres2[j] );
+#endif
 		totallen += nlen;
 #if 0
 		fprintf( stderr, "i=%d", i );
@@ -3402,6 +3236,7 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 	for( j=0; j<clus2; j++ ) result2[j][0] = 0;
 	totalscore = 0.0;
 	*fftlog = -1;
+//	reporterr( "\nin Falign_udpari(), *fftlog = %d\n", *fftlog );
 	for( i=0; i<count-1; i++ )
 	{
 		*fftlog += 1;
@@ -3549,6 +3384,10 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 		}
 #endif
 	}
+
+
+//	reporterr( "\nafter Falign_udpari(), *fftlog = %d\n", *fftlog );
+
 #if KEIKA
 	fprintf( stderr, "DP ... done   \n" );
 #endif
