@@ -871,7 +871,8 @@ double L__align11_noalign( double **n_dynamicmtx, char **seq1, char **seq2 )
 #if DEBUG
 		fprintf( stderr, "succeeded\n" );
 #endif
-		amino_dynamicmtx = AllocateDoubleMtx( 0x80, 0x80 );
+//		amino_dynamicmtx = AllocateDoubleMtx( 0x80, 0x80 ); 
+		amino_dynamicmtx = AllocateDoubleMtx( 0x100, 0x100 );  // 2017/Nov.  constants.c no 'charsize' wo global hensuu nishita houga yoi?
 		orlgth1 = ll1 - 100;
 		orlgth2 = ll2 - 100;
 	}
