@@ -1520,9 +1520,148 @@ void restorecommongaps( int njob, int n0, char **seq, int *ex1, int *ex2, int *g
 	free( tmpgapmap );
 }
 
-int deletenewinsertions_whole_eq( int on, int an, char **oseq, char **aseq, int **deletelist )
+#if 0 // mada
+int deletenewinsertions_difflist( int on, int an, char **oseq, char **aseq, GapPos **difflist )
 {
-	int i, j, p, q, allgap, ndel;
+	int i, j, p, q, allgap, ndel, istart, pstart;
+	int len = strlen( oseq[0] );
+	char *eqseq, tmpc;
+
+	reporterr( "In deletenewinsertions_difflist\n" );
+	for( j=0; j<on; j++ ) reporterr( "\no=%s\n", oseq[j] );
+	for( j=0; j<an; j++ ) reporterr( "a=%s\n", aseq[j] );
+
+
+	eqseq = calloc( len+1, sizeof( char ) );
+	for( i=0; i<len; i++ )
+	{
+		allgap = 0;
+		for( j=0; j<on; j++ )
+		{
+			tmpc = oseq[j][i];
+			if( tmpc != '-' && tmpc != '=' ) break;
+		}
+		if( j == on ) 
+			allgap = 1;
+
+		if( allgap )
+		{
+			eqseq[i] = '=';
+		}
+		else
+		{
+			eqseq[i] = 'o';
+		}
+	}
+
+	for( j=0; j<1; j++ ) reporterr( "\no                = %s\n", oseq[j] );
+	reporterr( "\ne                = %s\n", eqseq );
+	for( j=0; j<1; j++ ) reporterr( "a                = %s\n", aseq[j] );
+
+	if( difflist )
+	{
+		for( j=0; j<an; j++ )
+		{
+			ndel = 0;
+			for( i=0,q=0,p=0; i<len; i++ ) // 0 origin
+			{
+				tmpc = aseq[j][i];
+				if( eqseq[i] != '=' ) p++;
+				if( tmpc != '-' && tmpc != '=' ) q++;
+
+#if 0
+				if( eqseq[i] == '=' && ( tmpc != '-' && tmpc != '=' ) )
+				{
+					difflist[j] = realloc( difflist[j], sizeof( GapPos ) * (ndel+2) );
+					difflist[j][ndel].pos = -p;
+					difflist[j][ndel].len = 1;
+					ndel++;
+				}
+				if( eqseq[i] != '=' && ( tmpc == '-' || tmpc == '=' ) ) 
+				{
+//					reporterr( "deleting %d-%d, %c\n", j, i, aseq[j][i] );
+					difflist[j] = realloc( difflist[j], sizeof( GapPos ) * (ndel+2) );
+					difflist[j][ndel].pos = p;
+					difflist[j][ndel].len = 1;
+					ndel++;
+				}
+#else
+				istart = i;
+				pstart = p;
+				while( eqseq[i] == '=' && ( tmpc != '-' && tmpc != '=' ) )
+				{
+					i++;
+					tmpc = aseq[j][i];
+				}
+				if( i != istart )
+				{
+					difflist[j] = realloc( difflist[j], sizeof( GapPos ) * (ndel+2) );
+					difflist[j][ndel].pos = pstart;
+					difflist[j][ndel].len = -(i-istart);
+					ndel++;
+				}
+				istart = i;
+				pstart = p;
+				while( eqseq[i] != '=' && ( tmpc == '-' || tmpc == '=' ) ) 
+				{
+					i++;
+					tmpc = aseq[j][i];
+				}
+				if( i != istart )
+				{
+					difflist[j] = realloc( difflist[j], sizeof( GapPos ) * (ndel+2) );
+					difflist[j][ndel].pos = pstart;
+					difflist[j][ndel].len = i-istart;
+					ndel++;
+				}
+#endif
+			}
+			difflist[j][ndel].pos = -1;
+			difflist[j][ndel].len = 0;
+			for( i=0; ; i++ )
+			{
+				if( difflist[j][i].pos == -1 ) break;
+				reporterr( "sequence %d,%d, pos=%d,len=%d\n", j, i, difflist[j][i].pos, difflist[j][i].len );
+			}
+		}
+
+	}
+exit( 1 );
+	for( i=0,p=0; i<len; i++ )
+	{
+
+//		if( oseq[0][i] != '=' )
+//		reporterr( "i=%d, p=%d, q=%d, originally, %c\n", i, p, q, originallygapped[p]);
+//		if( eqseq[i] != '=' && originallygapped[p] != '-' ) // dame!!
+		if( eqseq[i] != '=' )
+		{
+//			reporterr( "COPY! p=%d\n", p );
+			if( p != i )
+			{
+				for( j=0; j<on; j++ ) oseq[j][p] = oseq[j][i];
+				for( j=0; j<an; j++ ) aseq[j][p] = aseq[j][i];
+			}
+			p++;
+		}
+	}
+//		reporterr( "deletemap        = %s\n", deletemap );
+//		reporterr( "eqseq            = %s\n", eqseq );
+//		reporterr( "originallygapped = %s\n", originallygapped );
+	for( j=0; j<on; j++ ) oseq[j][p] = 0;
+	for( j=0; j<an; j++ ) aseq[j][p] = 0;
+
+	free( eqseq );
+
+//	for( j=0; j<on; j++ ) reporterr( "\no=%s\n", oseq[j] );
+//	for( j=0; j<an; j++ ) reporterr( "a=%s\n", aseq[j] );
+
+	return( i-p );
+}
+#endif
+
+int deletenewinsertions_whole_eq( int on, int an, char **oseq, char **aseq, GapPos **deletelist )
+{
+	int i, j, p, q, allgap, ndel, qstart;
 	int len = strlen( oseq[0] );
 	char *eqseq, tmpc;
 
@@ -1551,6 +1690,7 @@ int deletenewinsertions_whole_eq( int on, int an, char **oseq, char **aseq, int
 			eqseq[i] = 'o';
 		}
 	}
+	eqseq[len] = 0; // hitsuyou
 
 //	for( j=0; j<1; j++ ) reporterr( "\no                = %s\n", oseq[j] );
 //	reporterr( "\ne                = %s\n", eqseq );
@@ -1564,19 +1704,40 @@ int deletenewinsertions_whole_eq( int on, int an, char **oseq, char **aseq, int
 			for( i=0,q=0; i<len; i++ )
 			{
 				tmpc = aseq[j][i];
+#if 0
 				if( tmpc != '-' && tmpc != '=' ) 
 				{
 					if( eqseq[i] == '=' )
 					{
 //						reporterr( "deleting %d-%d, %c\n", j, i, aseq[j][i] );
-						deletelist[j] = realloc( deletelist[j], sizeof( int ) * (ndel+2) );
-						deletelist[j][ndel] = q;
+						deletelist[j] = realloc( deletelist[j], sizeof( GapPos ) * (ndel+2) );
+						deletelist[j][ndel].pos = q;
+						deletelist[j][ndel].len = 1;
 						ndel++;
 					}
 					q++;
 				}
+#else
+				qstart = q;
+				while( (tmpc != '-' && tmpc != '=') && eqseq[i] == '=' )
+				{
+					i++;
+					q++;
+					tmpc = aseq[j][i];
+				}
+				if( qstart != q )
+				{
+//					reporterr( "deleting %d-%d, %c\n", j, i, aseq[j][i] );
+					deletelist[j] = realloc( deletelist[j], sizeof( GapPos ) * (ndel+2) );
+					deletelist[j][ndel].pos = qstart;
+					deletelist[j][ndel].len = q-qstart;
+					ndel++;
+				}
+				if( tmpc != '-' && tmpc != '=' ) q++;
+#endif
 			}
-			deletelist[j][ndel] = -1;
+			deletelist[j][ndel].pos = -1;
+			deletelist[j][ndel].len = 0;
 		}
 	}
 	for( i=0,p=0; i<len; i++ )
@@ -1610,9 +1771,10 @@ int deletenewinsertions_whole_eq( int on, int an, char **oseq, char **aseq, int
 	return( i-p );
 }
 
-int deletenewinsertions_whole( int on, int an, char **oseq, char **aseq, int **deletelist )
+#if 1 // 2022/Jan. disttbfast to tbfast niha hitsuyou.
+int deletenewinsertions_whole( int on, int an, char **oseq, char **aseq, GapPos **deletelist )
 {
-	int i, j, p, q, allgap, ndel;
+	int i, j, p, q, allgap, ndel, qstart;
 	int len = strlen( oseq[0] );
 	char *eqseq, tmpc;
 
@@ -1641,6 +1803,7 @@ int deletenewinsertions_whole( int on, int an, char **oseq, char **aseq, int **d
 			eqseq[i] = 'o';
 		}
 	}
+	eqseq[len] = 0;
 
 //	for( j=0; j<1; j++ ) reporterr( "\no                = %s\n", oseq[j] );
 //	reporterr( "\ne                = %s\n", eqseq );
@@ -1654,6 +1817,7 @@ int deletenewinsertions_whole( int on, int an, char **oseq, char **aseq, int **d
 			for( i=0,q=0; i<len; i++ )
 			{
 				tmpc = aseq[j][i];
+#if 0
 				if( tmpc != '-' ) 
 				{
 					if( eqseq[i] == '=' )
@@ -1665,8 +1829,27 @@ int deletenewinsertions_whole( int on, int an, char **oseq, char **aseq, int **d
 					}
 					q++;
 				}
+#else
+				qstart = q;
+				while( tmpc != '-' && eqseq[i] == '=' )
+				{
+					i++;
+					q++;
+					tmpc = aseq[j][i];
+				}
+				if( qstart != q )
+				{
+//					reporterr( "deleting %d-%d, %c\n", j, i, aseq[j][i] );
+					deletelist[j] = realloc( deletelist[j], sizeof( GapPos ) * (ndel+2) );
+					deletelist[j][ndel].pos = qstart;
+					deletelist[j][ndel].len = q-qstart;
+					ndel++;
+				}
+				if( tmpc != '-' ) q++;
+#endif
 			}
-			deletelist[j][ndel] = -1;
+			deletelist[j][ndel].pos = -1;
+			deletelist[j][ndel].len = 0;
 		}
 	}
 	for( i=0,p=0; i<len; i++ )
@@ -1701,7 +1884,6 @@ int deletenewinsertions_whole( int on, int an, char **oseq, char **aseq, int **d
 }
 
 
-#if 0
 int maskoriginalgaps( char *repseq, char *originallygapped )
 {
 	int i, p;
@@ -1800,9 +1982,9 @@ void restoreoriginalgaps( int n, char **seq, char *originalgaps )
 	free( tmpseq );
 }
 
-void reconstructdeletemap( int nadd, char **addbk, int **deletelist, char **realn, FILE *fp, char **name )
+void reconstructdeletemap( int nadd, char **addbk, GapPos **deletelist, char **realn, FILE *fp, char **name )
 {
-	int i, j, p, len;
+	int i, j, p, len, gaplen;
 	char *gapped, *nameptr, *tmpptr;
 
 	for( i=0; i<nadd; i++ )
@@ -1821,12 +2003,22 @@ void reconstructdeletemap( int nadd, char **addbk, int **deletelist, char **real
 		fprintf( fp, ">%s\n", nameptr );
 		fprintf( fp, "# letter, position in the original sequence, position in the reference alignment\n" );
 
+#if 0
 //		reporterr( "addbk[%d] = %s\n", i, addbk[i] );
 		for( j=0; (p=deletelist[i][j])!=-1; j++ )
 		{
 //			reporterr( "deleting %d, %c\n", p, addbk[i][p] );
 			gapped[p] = '-';
 		}
+#else
+//		reporterr( "addbk[%d] = %s\n", i, addbk[i] );
+		for( j=0; (p=deletelist[i][j].pos)!=-1; j++ )
+		{
+//			reporterr( "deleting %d, %c\n", p, addbk[i][p] );
+			gaplen = deletelist[i][j].len;
+			while( gaplen-- ) gapped[p++] = '-';
+		}
+#endif
 
 //		reporterr( "addbk  = %s\n", addbk[i] );
 //		reporterr( "gapped = %s\n", gapped );
@@ -1849,3 +2041,128 @@ void reconstructdeletemap( int nadd, char **addbk, int **deletelist, char **real
 		free( gapped );
 	}
 }
+
+#define MODIFYNAME 1 // MODIFYNAME mo --anysymbol to ryouritsu, 7.502-
+
+void reconstructdeletemap_compact( int nadd, char **addbk, GapPos **deletelist, char **realn, FILE *fp, char **name )
+{
+	int i, j, p, len, nins, gaplen;
+	char *gapped, *nameptr, *tmpptr;
+	int status = 0;
+
+#if MODIFYNAME
+	char *newname, *insstr;
+	newname = calloc( B+100, sizeof( char ) );
+	insstr = calloc( 1000, sizeof( char ) );
+#endif
+	fprintf( fp, "# Insertion in added sequence > Position in reference\n" );
+	for( i=0; i<nadd; i++ )
+	{
+		len = strlen( addbk[i] );
+		gapped = calloc( len+1, sizeof( char ) );
+//		reporterr( "len=%d", len );
+//		for( j=0; j<len; j++ ) gapped[j] = 'o'; // iranai
+//		gapped[len] = 0; // iranai
+
+		nameptr = name[i] + 1;
+		if( outnumber )
+			nameptr = strstr( nameptr, "_numo_e" ) + 8;
+
+		if( (tmpptr=strstr( nameptr, "_oe_" )) ) nameptr = tmpptr + 4; // = -> _ no tame
+
+		status = 0;
+
+
+#if 0
+//		reporterr( "addbk[%d] = %s\n", i, addbk[i] );
+		for( j=0; (p=deletelist[i][j])!=-1; j++ )
+		{
+//			reporterr( "deleting %d, %c\n", p, addbk[i][p] );
+			gapped[p] = '-';
+			status = 1;
+		}
+#else
+//		reporterr( "addbk[%d] = %s\n", i, addbk[i] );
+		for( j=0; (p=deletelist[i][j].pos)!=-1; j++ )
+		{
+//			reporterr( "deleting %d-%d, %c\n", p, p+deletelist[i][j].len, addbk[i][p] );
+			gaplen = deletelist[i][j].len;
+			while( gaplen-- ) gapped[p++] = '-'; // origin??????????? 2022/Jan
+			status = 1;
+		}
+#endif
+
+//		reporterr( "addbk  = %s\n", addbk[i] );
+//		reporterr( "gapped = %s\n", gapped );
+
+
+		if( status == 0 ) 
+		{
+			free( gapped );
+			continue;
+		}
+
+		fprintf( fp, ">%s\n", nameptr );
+
+		status = -1;
+#if MODIFYNAME
+		insstr[0] = 0;
+		nins = 0;
+#endif
+		for( j=0,p=0; j<len; j++ )
+		{
+			while( realn[i][p] == '-' )
+				p++;
+
+			if( gapped[j] == '-' )
+			{
+				if( status != 1 )	
+				{
+					status = 1;
+					fprintf( fp, "%d%c - ", j+1, addbk[i][j] ); // 1origin
+#if MODIFYNAME
+					if( nins == 0 )
+						sprintf( insstr, "%d%c-", j+1, addbk[i][j] );
+					else if( nins == 1 )
+						sprintf( insstr+strlen(insstr), "etc," );
+					nins++;
+#endif
+				}
+//				fprintf( fp, "%c, %d, -\n", addbk[i][j], j+1 ); // 1origin
+			}
+			else
+			{
+				if( status == 1 )	
+				{
+					fprintf( fp, "%d%c > %dv%d\n", j, addbk[i][j-1], p, p+1 ); // 1origin
+#if MODIFYNAME
+					if( nins == 1 ) sprintf( insstr+strlen(insstr), "%d%c,", j, addbk[i][j-1] );
+#endif
+				}
+				status = 0;
+//				fprintf( fp, "%c, %d, %d\n", addbk[i][j], j+1, p+1 ); // 1origin
+				p++;
+			}
+		}
+		if( status == 1 )	
+		{
+			fprintf( fp, "%d%c > %dv%d\n", j, addbk[i][j-1], p, p+1 ); // 1origin
+#if MODIFYNAME
+			if( nins == 1 ) sprintf( insstr+strlen(insstr), "%d%c,", j, addbk[i][j-1] );
+#endif
+		}
+		free( gapped );
+
+#if MODIFYNAME
+		insstr[strlen(insstr)-1] = 0;
+		strcpy( newname, name[i] );
+		sprintf( newname+(nameptr-name[i]), "%dins:%s|%s", nins, insstr, nameptr );
+		newname[B] = 0;
+		strcpy( name[i], newname );
+#endif
+	}
+#if MODIFYNAME
+	free( newname );
+	free( insstr );
+#endif
+}
