@@ -9,7 +9,6 @@
 static int treein;
 static int topin;
 static int treeout;
-static int distout;
 static int noalign;
 static int multidist;
 static int maxdist = 2; // scale -> 2bai
@@ -979,8 +978,15 @@ void arguments( int argc, char *argv[] )
 					addprofile = 0;
 					break;
 				case 'y':
-					distout = 1;
-					break;
+                                        distout = *(*++argv);
+					reporterr(       "distout=%c\n", distout );
+                                        if( distout != 'c' && distout != 'h' )
+                                        {
+                                            reporterr(       "Set -y c or -y h in v>=7.521.\n" );
+                                            exit( 1 );
+                                        }
+					--argc; 
+					goto nextoption;
 				case '^':
 					hitout = atof( *++argv );
 					--argc;
@@ -1070,6 +1076,9 @@ void arguments( int argc, char *argv[] )
 				case 'U':
 					treein = 1;
 					break;
+				case 's':
+					addtotop = 2;
+					break;
 				case 'x':
 					addtotop = 1;
 					break;
@@ -1838,8 +1847,10 @@ static void	*addsinglethread( void *arg )
 //			}
 //			fixed_musclesupg_double_realloc_nobk_halfmtx( njobc, iscorec, topolc, lenc, depc, 0, 1 );
 
-			if( addtotop )
-				neighbor = addonetip2top( njobc, topolc, lenc, iscorec, topol, len, dep, treeout, addtree, iadd, name, alnleninnode, nogaplenjusttodecideaddhereornot, noalign );
+			if( addtotop == 2 ) // root
+				neighbor = addonetip2fixedpos( njobc, topolc, lenc, iscorec, topol, len, dep, treeout, addtree, iadd, name, alnleninnode, nogaplenjusttodecideaddhereornot, noalign, 'r' );
+			else if( addtotop == 1 ) // top
+				neighbor = addonetip2fixedpos( njobc, topolc, lenc, iscorec, topol, len, dep, treeout, addtree, iadd, name, alnleninnode, nogaplenjusttodecideaddhereornot, noalign, 't' );
 			else
 				neighbor = addonetip( njobc, topolc, lenc, iscorec, topol, len, dep, treeout, addtree, iadd, name, alnleninnode, nogaplenjusttodecideaddhereornot, noalign );
 
@@ -2107,7 +2118,16 @@ void makecompositiontable_p( int *table, int *pointt )
 	int point;
 
 	while( ( point = *pointt++ ) != END_OF_VEC )
+	{
 		table[point]++;
+#if 0 // kakunin shinai
+		if( (unsigned int)table[point]++ >= INT_MAX )
+		{
+			reporterr( "Overflow. table[point]=%d>INT_MAX(%d).\n", table[point], INT_MAX );
+			exit( 1 );
+		}
+#endif
+	}
 }
 
 
