@@ -347,7 +347,7 @@ static int checkchar( int i )
 	return( 0 );
 }
 
-static void overridematrix( double **matrix )
+static int overridematrix( double **matrix )
 {
 	char buf[500];
 	FILE *fp;
@@ -355,6 +355,7 @@ static void overridematrix( double **matrix )
 	double v;
 	char *bpt;
 	int nread;
+	int userdefined;
 
 	fp = fopen( "_aamtx", "r" );
 	if( fp == NULL )
@@ -362,9 +363,10 @@ static void overridematrix( double **matrix )
 		fprintf( stderr, "warning: cannot open scorematrix. Use the default one.\n" );
 //		f2cl.c de tomaranai youni
 //		exit( 1 );
-		return;
+		return 0;
 	}
 
+	userdefined = 0;
 	while( 1 )
 	{
 		fgets( buf, 499, fp );
@@ -393,17 +395,20 @@ static void overridematrix( double **matrix )
 			reporterr( "%c=0x%x cannot be used (2)\n", i2, i2 );
 			exit( 1 );
 		}
-		reporterr( "Score(%c=0x%x,%c=0x%x)=%f\n", i1, i1, i2, i2, v );
+//		reporterr( "Score(%c=0x%x,%c=0x%x)=%f\n", i1, i1, i2, i2, v );
 		matrix[i1][i2] = v;
 		matrix[i2][i1] = v; // 2018/May/11
+		userdefined = 1;
 	}
 	fclose( fp );
+	return userdefined;
 }
 
-void extendedmtx( double **matrix, double *freq, unsigned char *amino, char *amino_grp )
+int extendedmtx( double **matrix, double *freq, unsigned char *amino, char *amino_grp )
 {
 	int i;
 	int j;
+	int userdefined;
 
 	for( i=0; i<nalphabets; i++ ) 
 	{
@@ -423,7 +428,8 @@ void extendedmtx( double **matrix, double *freq, unsigned char *amino, char *ami
 	for( i=0; i<nalphabets; i++ )
 		matrix[i][i] = matrix[i][i] = (double)1.0;
 
-	overridematrix( matrix );
+	userdefined = overridematrix( matrix );
+
 #if 0 // user-defined matrix + user-defined freqency ga aru toki fukkatsu saseru.
 	if( tmpmtx[400] != -1.0 ) 
 	{
@@ -441,4 +447,5 @@ void extendedmtx( double **matrix, double *freq, unsigned char *amino, char *ami
 		fprintf( stderr, "%d: %c, %d, %f\n", i, amino[i], amino_grp[amino[i]], freq[i] );
 	}
 #endif
+	return userdefined;
 }
