@@ -161,6 +161,47 @@ static void calcfreq( int nseq, char **seq, double *datafreq )
 	for( i=0; i<nscoredalphabets; i++ ) datafreq[i] /= (double)total;
 }
 
+
+void calcfreq_from_scoremtx( double **n_distmp, double *datafreq )
+{
+	int i, j;
+	int nused = 0;
+	for( i=0; i<nalphabets; i++ ) datafreq[i] = 0.0;
+	for( i=0; i<nalphabets; i++ ) for( j=0; j<i; j++ )
+	{
+		if( n_distmp[i][j] != -1 )
+		{
+			if( datafreq[i] == 0.0 ) nused += 1;
+			if( datafreq[j] == 0.0 ) nused += 1;
+			datafreq[i] = datafreq[j] = 1.0;
+		}
+	}
+        for( i=0; i<nalphabets; i++ ) datafreq[i] /= (double)nused;
+	reporterr( "nused=\n", nused );
+//	for( i=0; i<nalphabets; i++ ) reporterr( "%f\n", datafreq[i] );
+}
+
+static int checkscoremtx( double **n_distmp, int nseq, char **seq )
+{
+	int i, j, l, k;
+	int aan;
+	for( i=0; i<nseq; i++ )
+	{
+		l = strlen( seq[i] );
+		for( j=0; j<l; j++ )
+		{
+			aan = amino_n[(unsigned char)seq[i][j]];
+			for( k=0; k<nalphabets; k++ )
+			{
+//				if( n_distmp[k][aan] != -1.0 && k != aan ) break;
+				if( n_distmp[k][aan] != -1.0 ) break;
+			}
+			if( k == nalphabets ) return aan;
+		}
+	}
+	return 0;
+}
+
 static void calcfreq_extended( int nseq, char **seq, double *datafreq )
 {
 	int i, j, l;
@@ -606,6 +647,7 @@ void constants( int nseq, char **seq )
 		double average;
 //		double tmp;
 		double **n_distmp;
+		int userdefined;
 
 		nalphabets = 0x100;
 		nscoredalphabets = 0x100;
@@ -639,7 +681,7 @@ void constants( int nseq, char **seq )
 		penaltyLN = (int)( 600.0 / 1000.0 * -2000 + 0.5);
 		penalty_exLN = (int)( 600.0 / 1000.0 * -100 + 0.5);
 
-		extendedmtx( n_distmp, freq, amino, amino_grp );
+		userdefined = extendedmtx( n_distmp, freq, amino, amino_grp );
 
 		if( trywarp ) sprintf( shiftmodel, "%4.2f", -(double)penalty_shift/600 );
 		else sprintf( shiftmodel, "noshift" );
@@ -663,7 +705,18 @@ void constants( int nseq, char **seq )
 			freq1 = datafreq;
 		}
 		else
-			freq1 = freq;
+		{
+			calcfreq_from_scoremtx( n_distmp, datafreq );
+			freq1 = datafreq;
+		}
+#if 1
+		if( userdefined ) if( i=checkscoremtx( n_distmp, nseq, seq ) )
+		{
+			reporterr( "\n\nAlphabet %c (0x%x) is used in the sequence file but no score involving this alphabet is given in the matrix file.\n", i, i  );
+			reporterr( "Check if the data is as intended.\n\n\n" );
+			exit( 1 );
+		}
+#endif
 
 #if TEST
 		reporterr(       "raw scoreing matrix : \n" );
@@ -680,37 +733,43 @@ void constants( int nseq, char **seq )
 			average = 0.0;
 		else
 		{
-			for( i=0; i<nalphabets; i++ )
 #if TEST 
+			for( i=0; i<nalphabets; i++ )
 				fprintf( stdout, "freq[%c] = %f, datafreq[%c] = %f, freq1[] = %f\n", amino[i], freq[i], amino[i], datafreq[i], freq1[i] );
 #endif
 			average = 0.0;
 			for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ )
 				average += n_distmp[i][j] * freq1[i] * freq1[j];
 		}
-#if 1
+#if 0
 		if( disp ) fprintf( stdout, "####### average2  = %f\n", average );
 #endif
 
 		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) 
-			n_distmp[i][j] -= average;
-#if TEST
-		fprintf( stdout, "average2 = %f\n", average );
-		fprintf( stdout, "after average substruction : \n" );
+		{
+			if( n_distmp[i][j] == -1.0 ) 
+				n_distmp[i][j] = -average;
+			else
+				n_distmp[i][j] -= average;
+		}
+#if 0
+		fprintf( stderr, "average2 = %f\n", average );
+		fprintf( stderr, "after average subtruction : \n" );
 		for( i=0; i<nalphabets; i++ )
 		{
+			fprintf( stderr, "i=%d, %x\n", i, i );
 			for( j=0; j<nalphabets; j++ ) 
 			{
-				fprintf( stdout, "%6.2f", n_distmp[i][j] );
+				fprintf( stderr, "%6.2f", n_distmp[i][j] );
 			}
-			fprintf( stdout, "\n" );
+			fprintf( stderr, "\n" );
 		}
 #endif
 		
 		average = 0.0;
 		for( i=0; i<nalphabets; i++ ) 
 			average += n_distmp[i][i] * freq1[i];
-#if 1
+#if 0
 		if( disp ) fprintf( stdout, "####### average1  = %f\n", average );
 #endif
 
@@ -723,28 +782,30 @@ void constants( int nseq, char **seq )
 		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) 
 			n_distmp[i][j] *= 600.0 / average;
 #if TEST
-        fprintf( stdout, "after average division : \n" );
+        fprintf( stderr, "after average division : \n" );
         for( i=0; i<nalphabets; i++ )
         {
+            fprintf( stderr, "i=%d, %x\n", i, i );
             for( j=0; j<=i; j++ )
             {
-                fprintf( stdout, "%7.1f", n_distmp[i][j] );
+                fprintf( stderr, "%7.1f", n_distmp[i][j] );
             }
-            fprintf( stdout, "\n" );
+            fprintf( stderr, "\n" );
         }
 #endif
 
 		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) 
 			n_distmp[i][j] -= offset;  
 #if TEST
-		fprintf( stdout, "after offset substruction (offset = %d): \n", offset );
+		fprintf( stderr, "after offset subtruction (offset = %d): \n", offset );
 		for( i=0; i<nalphabets; i++ )
 		{
+			fprintf( stderr, "i=%d, %x\n", i, i );
 			for( j=0; j<=i; j++ ) 
 			{
-				fprintf( stdout, "%7.1f", n_distmp[i][j] );
+				fprintf( stderr, "%30.10f", n_distmp[i][j] );
 			}
-			fprintf( stdout, "\n" );
+			fprintf( stderr, "\n" );
 		}
 #endif
 #if 0
@@ -789,7 +850,7 @@ void constants( int nseq, char **seq )
 
 		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) n_dis[i][j] = 0;
 		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) n_dis[i][j] = (int)n_distmp[i][j];
-		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) n_dis[i][amino_n['-']] = n_dis[amino_n['-']][i] = 0.0;
+		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) n_dis[i][amino_n['-']] = n_dis[amino_n['-']][i] = 0;
 
 		FreeDoubleMtx( n_distmp );
 		FreeDoubleVec( datafreq );
@@ -900,7 +961,7 @@ void constants( int nseq, char **seq )
 			for( i=0; i<20; i++ ) for( j=0; j<20; j++ )
 				average += n_distmp[i][j] * freq1[i] * freq1[j];
 		}
-#if 1
+#if TEST
 		if( disp ) fprintf( stdout, "####### average2  = %f\n", average );
 #endif
 
@@ -925,7 +986,7 @@ void constants( int nseq, char **seq )
 		average = 0.0;
 		for( i=0; i<20; i++ ) 
 			average += n_distmp[i][i] * freq1[i];
-#if 1
+#if TEST
 		if( disp ) fprintf( stdout, "####### average1  = %f\n", average );
 #endif
 
