@@ -134,6 +134,10 @@ int maxl, tsize;
 char codonpos = 0;
 char codonscore = 0;
 
+char distout = 0;
+
+int terminalmargin = 100;
+
 
 void initglobalvariables()
 {
@@ -175,3 +179,4 @@ int specifictarget = 0;
 int nadd = 0; // <- static in tbfast.c, pairlocalalign.c
 int usenaivescoreinsteadofalignmentscore = 0;
 int nthreadreadlh = 1;
+int LineLengthInFASTA = -1;
