@@ -129,14 +129,25 @@ void stopolInit( int n, Node *stopol )
 
 void treeCnv( Node *stopol, int locnseq, int ***topol, double **len, double **bw )
 {
+	static int **tmpintvec = NULL;
+	static int numintvec = 0;
+	if( stopol == NULL )
+	{
+		while( --numintvec >= 0 )
+		{
+			free( tmpintvec[numintvec] );
+		}
+		free( tmpintvec );
+		numintvec = 0;
+		tmpintvec = NULL;
+		return;
+	}
 	int i;
 	NodeInCub parent;
 	int *count;
 	int ccount;
 	int rep;
 	int tmpint;
-	static int **tmpintvec = NULL;
-	static int numintvec = 0;
 
 	count = AllocateIntVec(  2 * locnseq ); /* oome */
 	if( !count ) ErrorExit( "Cannot allocate count.\n" );
